/*
 * Copyright (c) 2012 NINTENDO.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions, and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY NINTENDO AND ITS CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL NINTENDO OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

//
// interface for accessing xsock from WebKit
//

#pragma once

#include <string>
#include <vector>
#include <stdint.h>

class xsockClient;

namespace xsockInterface {

typedef std::string xsockString;
typedef uint64_t NetworkFileHandle;
const NetworkFileHandle invalidNetworkFileHandle = (NetworkFileHandle)-1;
const int operationWouldBlock = -2;

struct xsockCallbacks {
	int (*isNetworkHostFunc)(char const *host, xsockClient **socketOut);
	void (*closeNetworkHostFunc)(xsockClient *client);

	int (*fileExistsFunc)(xsockClient *socket, char const *path);
	int (*deleteFileFunc)(xsockClient *socket, char const *path);
	NetworkFileHandle (*openFileFunc)(xsockClient *socket, char const *path, const char *mode);
	void (*closeFileFunc)(xsockClient *socket, NetworkFileHandle* handle);
	long long (*seekFileFunc)(xsockClient *socket, NetworkFileHandle handle, long long offset, int origin);
	int (*truncateFileFunc)(xsockClient *socket, NetworkFileHandle handle, long long offset);
	int (*writeToFileFunc)(xsockClient *socket, NetworkFileHandle handle, const char* data, int length);
	int (*readFromFileFunc)(xsockClient *socket, NetworkFileHandle handle, char* data, int length);
	int (*deleteEmptyDirectoryFunc)(xsockClient *socket, char const *path);
	int (*getFileSizeFunc)(xsockClient *socket, char const *path, long long* result);
	int (*getFileModificationTimeFunc)(xsockClient *socket, char const *path, time_t* result);
	xsockString (*pathByAppendingComponentFunc)(xsockClient *socket, char const *path, const char *component);
	int (*makeAllDirectoriesFunc)(xsockClient *socket, char const *path);
	xsockString (*pathGetFileNameFunc)(xsockClient *socket, char const *path);
	xsockString (*directoryNameFunc)(xsockClient *socket, char const *path);
	xsockString (*fileSystemRepresentationFunc)(xsockClient *socket, char const *path);
	xsockString (*openTemporaryFileFunc)(xsockClient *socket, const char *prefix, NetworkFileHandle* handle);
	xsockString (*homeDirectoryPathFunc)(xsockClient *socket);
	xsockString (*localUserSpecificStorageDirectoryFunc)(xsockClient *socket);
	std::vector<xsockString> (*listDirectoryFunc)(xsockClient *socket, char const *path, const char *filter);
	int (*readFromFileNetwork_asyncSend)(xsockClient *socket, NetworkFileHandle handle, int length, int chunkSize);
	int (*readFromFileNetwork_asyncRecv)(xsockClient *socket, int id, char *data);
};

struct xsockParams {
	int param; // Not actually used but I'd rather not rip out the params only to have to add it back in later -LH-NSA 8/16/2012
};

void setParams(xsockParams *newParams);
void setLocalCallbacks(xsockCallbacks *newCallbacks);
void setNetworkCallbacks(xsockCallbacks *newCallbacks);

extern xsockParams params;
extern xsockCallbacks localCallbacks;
extern xsockCallbacks networkCallbacks;

}
