/*
 * Copyright (c) 2012 NINTENDO.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions, and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY NINTENDO AND ITS CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL NINTENDO OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef PLATFORMOS_H
#define PLATFORMOS_H

#include <cstring> // for size_t
#include <stdarg.h> // for va_list

namespace LibPlatform {
	namespace OS {
		typedef void *DynLoad_ModuleHandle;

#ifdef _WIIU_

		typedef signed long long Time;
		typedef unsigned Tick;

		/**
		Handle for sdk alarm structures
		*/
		class Alarm {
		public:
			Alarm(void);
			~Alarm(void);
			unsigned GetAlarmId(void);

		private:
			unsigned alarm_id;
		};

		typedef struct AtomicVar {
			union {
				unsigned u32;
				signed s32;
				void *ptr32;
			} u;
		} AtomicVar;

		struct Message {
			void *message;
			unsigned data0;
			unsigned data1;
			unsigned data2;
		};

		/**
		Handle for sdk message queues
		*/
		class MessageQueue {
		public:
			MessageQueue(void);
			~MessageQueue(void);
			unsigned GetMessageQueueId(void);

		private:
			unsigned message_queue_id;
		};

#endif // _WIIU_

		void SetReportCallback(int (*cb)(const char*, va_list));
		void Report(const char *format, ...);
		void VReport(const char *format, va_list args);
		unsigned GetConsoleType(void);
		signed DynLoad_Acquire(char const *module_name, void *module_handle);
		signed DynLoad_FindExport(void *module_handle, signed type_is_data, char const *export_name, void **return_address);
		void DynLoad_Release(void *module_handle);

#ifdef _WIIU_

		Tick MillisecondsToTicks(double milliseconds);
		void SleepMilliseconds(double milliseconds);
		void SleepMicroseconds(double milliseconds);
		void CreateAlarm(Alarm *alarm);
		void CancelAlarm(Alarm *alarm);
		void SetAlarm(Alarm *alarm, Time ticks_delay, void (*)(void *, void *));
		signed IncAtomic(volatile AtomicVar *pointer);
		signed DecAtomic(volatile AtomicVar *pointer);
		unsigned GetCoreId(void);
		void GetCodegenVirtAddrRange(unsigned *codegenva, size_t *codegensize);
		void SwitchSecCodeGenMode(bool bExecute);
		void GetStack(void **base, void **end);
		void InitMessageQueue(MessageQueue *message_queue, Message *message, signed count);
		signed SendMessage(MessageQueue *message_queue, Message *message, signed flags);
		signed ReceiveMessage(MessageQueue *message_queue, Message *message, signed flags);

#endif // _WIIU_
	}
}

#endif // PLATFORMOS_H
