/*
 * Copyright (c) 2012 NINTENDO.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions, and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY NINTENDO AND ITS CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL NINTENDO OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef PLATFORMSSL_H
#define PLATFORMSSL_H

namespace LibPlatform {
	class SSL {
	public:
		typedef enum {
			SSL_SSLVERSION_INVALID = -1,

			/* Automatically select the highest version supported out of TLS1.0/SSLv3 */
			SSL_SSLVERSION_AUTO,

			/* TLS 1.0 */
			SSL_SSLVERSION_TLSv1,

			/* SSLv3 */
			SSL_SSLVERSION_SSLv3
		} SSLSSLVersion;

		typedef enum {
		   SSL_CERT_TYPE_DER
		} SSLCertType;

		typedef signed SSLContextHandle;

		static signed Init(void);
		static signed CreateContext(SSLSSLVersion ssl_version);
		static signed DestroyContext(signed context);
		static signed AddServerPKIExternal(
			SSLContextHandle context,
			const unsigned char *pServerCert, 
			int serverCertLen,
			SSLCertType serverCertType
		);
	};
}

#endif//PLATFORMSSL_H
