/*
 * Copyright (c) 2012 NINTENDO.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions, and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY NINTENDO AND ITS CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL NINTENDO OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _WKGFX_MISC_H_
#define _WKGFX_MISC_H_

#include <config.h>

class PlatformInput;

namespace WebCore {
	class AffineTransform;
	class ShadowBlur;
	class FloatPoint;
	class FloatSize;
	class FloatRect;
	class IntRect;
	class Color;
	class Path;
	class FontPlatformData;
	class Font;
	class FontData;
	class FontFallbackList;
	class SimpleFontData;
	class MediaPlayer;
	class MediaPlayerPrivateWiiU;
	class GraphicsContext;
	struct PlatformMedia;
	class TimeRanges;
};

namespace WTF {
	class String;
	class StatisticsWiiU;
};

class MediaTimer;

struct font_param;
struct font_config_handle;
struct font_face_t;

#include <ColorSpace.h>
#include <PassRefPtr.h>
#include <ByteArray.h>
#include <CString.h>
#include <FontCache.h>
#include <Timer.h>
#include <TimeRanges.h>
#include <string>

namespace WKgfx {

// ----------------------------------------------------------------------------------

typedef void (*CBpostDLCall)(void *pDLaddr, unsigned int dlSize, unsigned long long *pTimeStamp, unsigned long long timeStampValue, bool bDrawSync);

// ----------------------------------------------------------------------------------

void RenderWebkitDL(void * pAddr, unsigned int size, unsigned long long timeStampVal, unsigned long long * pTimeStamp, bool bDrawSync);

// ----------------------------------------------------------------------------------

//! Interface to facilitate callbacks into Wekbkit without causing true circular dependencies
class CallbackHooks
{
public:
	// WebCore::ShadowBlur
	virtual void ShadowBlur_ShadowBlur(WebCore::ShadowBlur *pThis) = 0;
	virtual void ShadowBlur_setShadowValues(WebCore::ShadowBlur *pThis, const WebCore::FloatSize&, const WebCore::FloatSize& , const WebCore::Color&, WebCore::ColorSpace, bool ignoreTransforms) = 0;

	// WTF::StatisticsWiiU
	virtual WTF::StatisticsWiiU *StatisticsWiiU_instance(void) = 0;

	// WebCore::Path
	virtual void Path_Path(WebCore::Path *pThis) = 0;
	virtual void Path_DestructPath(WebCore::Path *pThis) = 0;
	virtual void Path_addArc(WebCore::Path *pPath, const WebCore::FloatPoint&, float radius, float startAngle, float endAngle, bool anticlockwise) = 0;
	virtual void Path_moveTo(WebCore::Path *pPath, const WebCore::FloatPoint &) = 0;
	virtual void Path_addLineTo(WebCore::Path *pPath, const WebCore::FloatPoint &) = 0;
	virtual void Path_addQuadCurveTo(WebCore::Path *pPath, const WebCore::FloatPoint &, const WebCore::FloatPoint &) = 0;
	virtual void Path_addBezierCurveTo(WebCore::Path *pPath, const WebCore::FloatPoint &, const WebCore::FloatPoint &, const WebCore::FloatPoint &) = 0;

	// WebCore::AffineTransform
	virtual WebCore::FloatPoint AffineTransform_mapPoint(const WebCore::AffineTransform *pThis, const WebCore::FloatPoint& point) = 0;
	
	// WebCore::IntRect
	virtual void IntRect_intersect(WebCore::IntRect *pThis, const WebCore::IntRect & rect) = 0;
	virtual void IntRect_IntRect(WebCore::IntRect *pThis, const WebCore::FloatRect & rect) = 0;

	// WebCore::FloatRect
	virtual void FloatRect_fitToPoints(WebCore::FloatRect *pThis, const WebCore::FloatPoint& p0, const WebCore::FloatPoint& p1, const WebCore::FloatPoint& p2, const WebCore::FloatPoint& p3) = 0;
	virtual void FloatRect_FloatRect(WebCore::FloatRect *pThis, const WebCore::IntRect & rect) = 0;

	// WTF::ByteArray
	virtual WTF::PassRefPtr<WTF::ByteArray> ByteArray_create(size_t size) = 0;

	// WTF::String
    virtual WTF::CString String_ascii(const WTF::String *pThis) = 0;
	
	// WTF::Mutex
	virtual bool Mutex_tryLock(Mutex *pThis) = 0;
	virtual void Mutex_unlock(Mutex *pThis) = 0;
	
	// WTF
	virtual WTF::ThreadIdentifier WTF_currentThread(void) = 0;
	virtual bool WTF_isMainThread(void) = 0;
	
	// WebCore::FontFallbackList_fontDataAt
	virtual const WebCore::FontData* FontFallbackList_fontDataAt(const WebCore::FontFallbackList *pThis, const WebCore::Font* pFont, unsigned index) = 0;

	// WebCore::FontCache
	virtual WebCore::SimpleFontData* FontCache_getCachedFontData(WebCore::FontCache *pThis, const WebCore::FontPlatformData *pData, WebCore::FontCache::ShouldRetain bRetain) = 0;
	
	// WebCore::FontPlatformData
	virtual void FontPlatformData_FontPlatformData(WebCore::FontPlatformData *pThis, font_config_handle *pHandle, font_param *pParam, const WebCore::FontDescription & desc) = 0;
	virtual void FontPlatformData_FontPlatformData(WebCore::FontPlatformData *pThis, font_face_t *face, float size, bool bBold, bool bItalic) = 0;
	virtual void FontPlatformData_FontPlatformData(WebCore::FontPlatformData *pThis, const WebCore::FontPlatformData & data) = 0;
	virtual void FontPlatformData_DestructFontPlatformData(WebCore::FontPlatformData *pThis) = 0;
	
	// WebCore::MediaPlayerPrivateWiiU
	virtual WebCore::Timer<MediaTimer>* MediaPlayerPrivateWiiU_CreateTimer(MediaTimer *pTimer) = 0;
	virtual void MediaPlayerPrivateWiiU_DestroyTimer(WebCore::Timer<MediaTimer>* pTimer) = 0;
	virtual void MediaPlayerPrivateWiiU_StartTimer(WebCore::Timer<MediaTimer>* pTimer, double period) = 0;
	virtual void MediaPlayerPrivateWiiU_StopTimer(WebCore::Timer<MediaTimer>* pTimer) = 0;
	virtual int MediaPlayerPrivateWiiU_getDisplayIndex(const WebCore::MediaPlayerPrivateWiiU *pThis) = 0;
	virtual bool MediaPlayerPrivateWiiU_isURLallowable(const std::string & url) = 0;
	virtual void MediaPlayerPrivateWiiU_doLoad(WebCore::MediaPlayerPrivateWiiU *pThis, const WTF::String & url) = 0;

	// WebCore::MediaPlayer
	virtual void MediaPlayer_networkStateChanged(WebCore::MediaPlayer *pThis) = 0;
	virtual void MediaPlayer_readyStateChanged(WebCore::MediaPlayer *pThis) = 0;
	virtual void MediaPlayer_timeChanged(WebCore::MediaPlayer *pThis) = 0;
	virtual void MediaPlayer_repaint(WebCore::MediaPlayer *pThis) = 0;

	// WebCore::TimeRanges
	virtual void TimeRanges_add(WebCore::TimeRanges *pRange, float start, float end) = 0;
	virtual bool TimeRanges_contain(const WebCore::TimeRanges *pRange, float time) const = 0;
	virtual float TimeRanges_nearest(const WebCore::TimeRanges *pRange, float time) const = 0;

	// WebCore::GraphicsContext
	virtual void GraphicsContext_fillRect(WebCore::GraphicsContext *pThis, const WebCore::FloatRect& r, const WebCore::Color& c, WebCore::ColorSpace cs) = 0;
	virtual bool GraphicsContext_paintingDisabled(const WebCore::GraphicsContext *pThis) = 0;
	
	// WebCore
	virtual const WebCore::PlatformMedia & getNoPlatformMedia() = 0;

	// Assertions
	virtual void WebKitCrash(void) = 0;

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	
	static void setInstance(CallbackHooks *pInstance)	{ s_pInstance = pInstance; }
	static CallbackHooks* getInstance(void)				{ return s_pInstance; }
	
private:
	static CallbackHooks *		s_pInstance;
};

// ----------------------------------------------------------------------------------

} // namespace WKgfx

#endif // _WKGFX_CLIPOP_H_
